import { ItemStack, MinecraftItemTypes, world } from "mojang-minecraft";
import EntityDB from './entity_database.js';
import InventoryObserver from './inventory_observer.js';

/*================================
	Todos os direitos reservados a @evil_makima (https://twitter.com/evil_makima?t=yu6yLMmVLyTsiQy09TP0GQ&s=09).
	Você pode ser processado se pegar algo deste add-on '///'
	
	All rights reserved to @evil_makima (https://twitter.com/evil_makima?t=yu6yLMmVLyTsiQy09TP0GQ&s=09)..
	You can be sued if you get something from this add-on :)
	
	Todos los derechos reservados a @evil_makima (https://twitter.com/evil_makima?t=yu6yLMmVLyTsiQy09TP0GQ&s=09).
	Puede ser demandado si obtiene algo de este complemento '///'
	
	@evil_makima (https://twitter.com/evil_makima?t=yu6yLMmVLyTsiQy09TP0GQ&s=09) 保留所有权利。
	如果你从这个插件中得到了一些东西，你可能会被起诉 '///'
================================*/

const shared_furnace_recipes = {
	"raw_copper_block": { id: "copper_block", data: 0},
	"raw_iron_block": { id: "iron_block", data: 0},
	"raw_gold_block": { id: "gold_block", data: 0},
	
	"quartz_ore": { id: "quartz", data: 0},
	"ancient_debris": { id: "netherite_scrap", data: 0},
	
	"coal_ore#deepslate_coal_ore": { id: "coal", data: 0},
	"diamond_ore#deepslate_diamond_ore": { id: "diamond", data: 0},
	"emerald_ore#deepslate_emerald_ore": { id: "emerald", data: 0},
	"lapis_ore#deepslate_lapis_ore": { id: "lapis_lazuli", data: 0},
	"redstone_ore#deepslate_redstone_ore": { id: "redstone", data: 0},
	
	"copper_ore#deepslate_copper_ore#raw_copper": { id: "copper_ingot", data: 0},
	"gold_ore#deepslate_gold_ore#raw_gold#nether_gold_ore": { id: "gold_ingot", data: 0},
	"iron_ore#deepslate_iron_ore#raw_iron": { id: "iron_ingot", data: 0},
	
	"red_sand#sand": { id: "glass", data: 0},
	"basalt": { id: "smooth_basalt", data: 0},
	
	"clay": { id: "brick", data: 0},
	"clay_block": { id: "hardned_clay", data: 0},
	
	"hardned_clay": { id: "glazed_terracota", data: "current" },
	"log": { id: "charcoal", data: "contains" },
	
	"beef": { id: "cooked_beef", data: 0},
	"chicken": { id: "cooked_chicken", data: 0},
	"cod": { id: "cooked_cod", data: 0},
	"porkchop": { id: "cooked_porkchop", data: 0},
	"rabbit": { id: "cooked_rabbit", data: 0},
	"salmon": { id: "cooked_salmon", data: 0},
	"chorus_fruit": { id: "popped_chorus_fruit", data: 0},
	"potato": { id: "baked_potato", data: 0},
	"mutton": { id: "cooked_mutton", data: 0},
	
	"cobblestone": { id: "stone", data: 0},
	"stone": { id: "smooth_stone", data: 0},
	"netherrack": { id: "netherbrick", data: 0},
	"deepslate": { id: "cobbled_deepslate", data: 0}
	
};

const common_fuel = {
	"minecraft:coal": { "max_data": 10 },
	"minecraft:coal_block": { "max_data": 90 },
	"minecraft:charcoal": { "max_data": 10 },
	"minecraft:charcoal_block": { "max_data": 90 },
	"minecraft:bucket": { "max_data": 900, "fuel_data": 2},
	"minecraft:blaze_rod": { "max_data": 10 },
	"minecraft:stick": { "max_data": 1 }
};

const common_fuel_material = {
	"wood": { "max_data": 2 },
	"planks": { "max_data": 2 },
	"log": { "max_data": 8 },
	"stick": { "max_data": 1 },
	"coal": { "max_data": 8 }
};

const default_instance = {
	progression: 0,
	heat: 0,
	xp: 0,
	multiplier: 1
};

class EntityBench {
	constructor(){
		this.list = {};
		this.shareds = {
			fuels: common_fuel,
			fuels_materials: common_fuel_material,
			recipes: {}
		};
		
		Object.keys(shared_furnace_recipes).forEach( recipes => {
			recipes.split("#").forEach( recipe => {
				this.shareds.recipes[`minecraft:${(recipe)}`] = shared_furnace_recipes[recipes];
			});
		});
	};
	
	register( id, name, data, heat){
		this.list[id] = {
			data: data,
			name: name,
			heat: heat??0,
			upgrades: {},
			events: {},
			instances: {}
		};
		
		this.current = id;
		
		return this;
	};
	
	registerEvent( name, callback){
		if(this.current)
			this.list[this.current].events[name] = callback;
		return this;
	};
	
	registerUpgrade( id, data){
		if(this.current)
			this.list[this.current].upgrades[id] = data;
		
		return this;
	};
	
	getFuel( entity, stack){
		if(!stack) return 0;
		
		let fuel = this.shareds.fuels[stack.id]?this.shareds.fuels[stack.id].max_data:0;
		
		if(fuel == 0){
			Object.keys(this.shareds.fuels_materials).forEach( ( material, index) => {
				if((stack.id).includes(material))
					fuel = this.shareds.fuels_materials[material].max_data??0;
			});
		};
		
		return fuel;
	};
	
	getRecipe(stack){
		return this.shareds.recipes[stack.id]??0;
	};
	
	getSlot( container, index){
		return container.getItem(index) ?? { id:"minecraft:air", data:0, amount: 0};
	};
	
	setInstanceValue( entity, key, value){
		this.getInstance( entity )[key] = value;
	};
	
	setInstance( entity, value){
		let { x, y, z} = entity.location;
		this.list[entity.id].instances[`${x}:${y}:${z}`] = value;
	};

	getInstance( entity ){
		let { x, y, z} = entity.location;
		
		if(!this.list[entity.id].instances[`${x}:${y}:${z}`])
			this.list[entity.id].instances[`${x}:${y}:${z}`] = { ...default_instance };
		
		return this.list[entity.id].instances[`${x}:${y}:${z}`];
	};
	
	craft( entity ){
		if(this.list[entity.id]){
			let instance = this.getInstance( entity );
			let { progression, heat, multiplier, xp} = instance;

			if(entity.nameTag.length < 3){
				entity.runCommand(`replaceitem entity @s slot.inventory 1 makima:progress_0 1 0`);
				entity.runCommand(`replaceitem entity @s slot.inventory 0 makima:fuel_0 1 0`);
				
				if(this.list[entity.id].events["on_setup"])
					this.list[entity.id].events["on_setup"](entity);
			};
			
			entity.nameTag = instance.fantasy_name??this.list[entity.id].name;
			
			let fuel = EntityDB.getInt( entity, "fuel" );
			let max_fuel = EntityDB.getInt( entity, "max_fuel" );
			
			let container = entity.getComponent("inventory").container;
			let contains_materials = false;
			
			let { fuels, inputs, outputs } = this.list[entity.id].data;
			
			//upgrade system start
			try {
				if(this.list[entity.id].data.upgrades){
					let hasup = {};
					
					this.list[entity.id].data.upgrades.forEach( upindex => {
						let stack = this.getSlot( container, upindex);
						
						if(this.list[entity.id].upgrades[stack.id]){
							hasup[stack.id] = true;
							this.list[entity.id].upgrades[stack.id].matching(entity);
						}
					});
					
					Object.keys(this.list[entity.id].upgrades).forEach( upid => {
						if(!hasup[upid])
							this.list[entity.id].upgrades[upid].unmatched(entity);
					});
				}
			} catch( error ){
				entity.runCommand(`say error: ${error}`);
			};
			//upgrade system end
			
			if(fuel > 0){
				let pp = 0, pu = 0;
				
				if(progression > 0)
					 pp = (Math.floor((progression/(10-(this.list[entity.id].heat+heat)))*5.0));
				
				inputs.forEach((e,i) => {
					try {
						let stack = this.getSlot( container, e);
						let stack_out = this.getSlot( container, outputs[i]);
						let recipe = this.getRecipe( stack );
						
						let condition_00 = typeof(recipe)==typeof({});
						let condition_01 = (condition_00 && stack_out.id.split(":")[1] == recipe.id && stack_out.data == recipe.data);
						let condition_02 = (condition_00 && stack_out.id == "minecraft:air");
						let condition_03 = (condition_00 && (stack_out.amount+((recipe.amount??1))*multiplier)<=(recipe.max_amount??64))
						let condition_04 = (condition_01 || condition_02);
						
						if(condition_03 && condition_04){
							pu++;
							
							if(progression > (9-this.list[entity.id].heat)){
								if(stack.amount > 1) {
									entity.runCommand(`replaceitem entity @s slot.inventory ${e} ${(stack.id)} ${(stack.amount-1)} ${(stack.data)}`);
									contains_materials = true;
								} else 
									container.setItem( e, new ItemStack(MinecraftItemTypes["air"], 1, 0));
								
								let fdata = (recipe.data=="current")?stack.data:recipe.data;
								
								entity.runCommand(`replaceitem entity @s slot.inventory ${(outputs[i])} ${(recipe.id)} ${(stack_out.amount+((recipe.amount??1)*multiplier))} ${(fdata)}`);
								
								if(xp > 0)
									entity.runCommand(`xp ${xp*(multiplier??1)} @p`);
								
								if(this.list[entity.id].events["on_craft"])
									this.list[entity.id].events["on_craft"](entity);
								
							}
						}
					} catch( e ){
						entity.runCommand(`say error: ${e}`);
					};
				});
				
				if(progression > (9-this.list[entity.id].heat)){
					progression = 0;
					
					if(contains_materials)
						EntityDB.removeInt( entity, "fuel", 1);
				};
				
				if(pu > 0){
					progression++;
					entity.runCommand(`replaceitem entity @s slot.inventory 1 makima:progress_${(pp<5?pp:4)} 1 0`);
				};
				
				try {
					entity.runCommand(`replaceitem entity @s slot.inventory 0 makima:fuel_${(Math.floor((fuel/max_fuel)*4))} 1 0`);
				}catch(error){
					entity.runCommand(`say error: ${error}`);
				}
			} else {
				let stack = this.getSlot( container, fuels);
				let new_fuel = this.getFuel( entity, stack);
				
				if(new_fuel < 1) return;
				
				EntityDB.setIntList( entity, {
					fuel: new_fuel,
					max_fuel: new_fuel
				});
				
				if(stack.amount > 1) {
					entity.runCommand(`replaceitem entity @s slot.inventory ${fuels} ${(stack.id)} ${(stack.amount-1)} ${(stack.data)}`);
				} else {
					container.setItem( fuels, new ItemStack(MinecraftItemTypes["air"], 1, 0));
				};
				
				fuel = new_fuel;
				max_fuel = new_fuel;
			};
			
			this.setInstanceValue( entity, "progression", progression);
		}
	}
};

const delete_matcher = ( target, item, container, index) => {
	container.setItem( index, new ItemStack(MinecraftItemTypes.air, 1, 0));
};
 
[ "progress", "fuel"].forEach( a => {
	for(var b = 0; b < 5; b++){
		InventoryObserver.register( `makima:${a}_${b}`, {
			on_matching: delete_matcher
		})
	}
});

const entity_bench = new EntityBench();

world.events.effectAdd.subscribe( data => {
	try {
		entity_bench .craft( data.entity );
	} catch(error) {
		data.entity.runCommand(`say ${error}`);
	}
});

export default entity_bench;