import { BlockLocation, world } from "mojang-minecraft";
import EntityDB from './entity_database.js';

class EntityMachinery {
	constructor(){
		this.list = {};
		this.instances = {};
	}
	
	register( id, prototype){
		this.list[id] = prototype;
	}
	
	getCoordinatesID( entity ){
		let { x, y, z} = entity.location;
		
		x = Math.floor(x);
		y = Math.floor(y);
		z = Math.floor(z);
		
		return `${x}:${y}:${z}`;
	}
	
	getInstanceFromCoordinates( x, y, z){
		let cid = `${x}:${y}:${z}`;
		
		return this.instances[cid];
	}
	
	getInstance( entity ){
		let cid = this.getCoordinatesID( entity );
		
		this.instances[cid] = this.instances[cid] ?? { ... (this.list[entity.id].data) };
		this.instances[cid].entity = entity;
		
		return this.instances[cid];
	}
	
	instance( entity ){
		return {
			setDataValue: ( key, value) => {
				this.getInstance( entity );
				
				this.instances[this.getCoordinatesID( entity )][key] = value;
				
				return this.instance( entity );
			},
			getDataValue: ( key, value) => {
				this.getInstance( entity );
				
				return { value: this.instances[this.getCoordinatesID( entity )][key], instance: this.instance( entity ) };
			},
			getInt( key ){
				return { value: EntityDB.getInt( entity, key), instance: this.instance( entity ) };
			},
			addInt( key, value){
				EntityDB.addInt( entity, key, value);
				return this.instance( entity );
			},
			removeInt( key, value){
				EntityDB.removeInt( entity, key, value);
				return this.instance( entity );
			},
			setInt( key, value){
				EntityDB.setInt( entity, key, value);
				return this.instance( entity );
			},
			setIntList( data ){
				EntityDB.setIntList( entity, data);
				return this.instance( entity );
			},
			addIntList( data ){
				EntityDB.addIntList( entity, data);
				return this.instance( entity );
			},
			removeIntList( data ){
				EntityDB.removeIntList( entity, data);
				return this.instance( entity );
			}
		}
	}
	
	tick( entity ){
		if(!this.list[entity.id] || !this.list[entity.id].tick) return;
		
		this.list[entity.id].tick( entity, this.getInstance( entity ));
	}
};

const entity_machinery = new EntityMachinery();

world.events.effectAdd.subscribe( data => {
	entity_machinery .tick( data.entity );
});

export default entity_machinery;