import { world } from "mojang-minecraft";

class InventoryObserver {
	constructor(){
		this.delay = 1000/10;
		
		this.list = {};
		this.timers = {};
		this.matching = false;
	};
	
	register( id, matcher){
		this.list[id] = matcher;
		this.list[id].range = [ 0, 36];
	};
	
	hasMatch( item ){
		let matcher = false;
		
		if(item && this.list[item.id]){
			matcher = this.list[item.id];

			[ "data", "amount"].forEach( m => {
				if(matcher["max_"+m] && matcher["max_"+m] < item[m])
					matcher = false;
				if(matcher["min_"+m] && matcher["min_"+m] > item[m])
					matcher = false;
				if(matcher[m] && matcher[m] != item[m])
					matcher = false;
			})
		};
		
		return matcher;
	};
	
	getItem( container, index){
		var out_slot = container.getItem(index);
		
		return (out_slot != null)?out_slot:{ id: "minecraft:air", amount: 0, data: 0};
	};

	tryMatch( target, container, index){
		let item = this.getItem( container, index);
		let match = this.hasMatch( item );
		
		if(match){
			let range = this.list[item.id].range;
			
			if(index >= range[0] || index <= range[1]){
				let cn1 = (target.selectedSlot == index && match.hand);
				let cn2 = (target.selectedSlot != index && !match.hand);
				
				if(cn1 || cn2)
					this.list[item.id].on_matching( target, item, container, index);
			}
		}
	};
	
	delayed( id, ms, fun){
		let time = Date.now();
		if(!this.timers[id]) this.timers[id] = time;
		
		if(time > this.timers[id]+ms){
			fun(time);
			this.timers[id] = time;
		}
	}
};

const observer = new InventoryObserver();

world.events.tick.subscribe(() => {
	observer.delayed( "makima:iot", observer.delay, time => {
		world.getPlayers().forEach( player => {
			let container = player.getComponent("inventory").container;
			for(var index = 0; index < 36; index++){
				observer.tryMatch( player, container, index);
			}
		});
	});
});

export default observer;